<?php
final class UsersAdmin extends Users{

  private static $oInstance = null;
  public $aPriceGroups = null;
  public $aUsers = null;

  public static function getInstance( $config = null ){  
    if( !isset( self::$oInstance ) ){  
      self::$oInstance = new UsersAdmin( );  
    }  
    return self::$oInstance;  
  } // end function getInstance

  /**
  * Constructor
  * @return void
  */
  private function __construct( ){
    if( !isset( $this->aGroups ) ){
      $oFFS = FlatFilesSerialize::getInstance( );
      $this->aGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    }
  } // end function __construct

  /**
  * Lists price groups
  * @return string
  * @param array $aPrices
  * @param string $mOption
  */
  public function listPriceGroups( $aPrices, $mOption = null ){
    $oFFS = FlatFilesSerialize::getInstance( );
    if( isset( $this->aPriceGroups ) )
      $aGroups = $this->aPriceGroups;
    elseif( isset( $this->aGroups ) )
      $aGroups = $this->aGroups;
    else
      $aGroups = $this->aGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    $content = null;

    if( isset( $aGroups ) && is_array( $aGroups ) && count( $aGroups ) > 0 ){
      $i = 0;
      foreach( $aGroups as $iGroup => $aList ){
        if( $i < PRICE_GROUPS_LIMIT && $aList['sLang'] == LANGUAGE && !isset( $aList['iPriceGroup'] ) ){
          $this->aPriceGroups[$iGroup] = $this->aGroups[$iGroup];
          $aList['fPrice'] = isset( $aPrices[$iGroup] ) ? $aPrices[$iGroup] : ( isset( $aList['fDiscount'] ) ? $aList['fDiscount'].'%' : null );
          if( isset( $mOption ) && is_numeric( $mOption ) ) // mOption = iProduct
            $content .= '<li><input type="text" name="aPricesGroups['.$mOption.']['.$aList['iGroup'].']" value="'.$aList['fPrice'].'" class="inputr" size="7" /></li>';
          elseif( $mOption == 'form' )
            $content .= '<label for="price'.$aList['iGroup'].'">'.$aList['sName'].'</label>:&nbsp;<input type="text" name="aPrices['.$aList['iGroup'].']" value="'.$aList['fPrice'].'" id="price'.$aList['iGroup'].'" class="inputr" size="10" />';
          else
            $content .= '<li>'.$aList['sName'].'</li>';
          $i++;
          if( $i == PRICE_GROUPS_LIMIT )
            break;
        }
      } // end for
    }
    if( isset( $content ) ){
      if( $mOption == 'form' )
        return '<span class="pricesList">'.$content.'</span>';
      else
        return '<ul>'.$content.'<ul>';
    }
  } // end function listPriceGroups

  /**
  * Saves related groups to user or pages
  * @return void
  * @param int  $iRelated
  * @param array $aRelated
  * @param string $sKey
  */
  public function saveRelatedToGroups( $iRelated, $aRelated = null, $sKey = 'Users' ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    if( isset( $aRelated ) && is_array( $aRelated ) )
      $aRelated = array_flip( $aRelated );
    if( isset( $aUsersGroups ) && is_array( $aUsersGroups ) && count( $aUsersGroups ) > 0 ){
      foreach( $aUsersGroups as $iGroup => $aGroup ){
        if( $aGroup['sLang'] == LANGUAGE ){
          if( isset( $aRelated[$iGroup] ) ){
            if( !isset( $aGroup['a'.$sKey][$iRelated] ) ){
              $aUsersGroups[$iGroup]['a'.$sKey][$iRelated] = 1;
              $bChanged = true;
            }
          }
          elseif( isset( $aGroup['a'.$sKey][$iRelated] ) ){
            unset( $aUsersGroups[$iGroup]['a'.$sKey][$iRelated] );
            $bChanged = true;
          }
        }
      } // end for
      if( isset( $bChanged ) )
        $oFFS->saveData( DB_USERS_GROUPS, $aUsersGroups );
    }
  } // end function saveRelatedToGroups

  /**
  * Shows users list
  * @return string
  */
  public function listUsers( ){
    global $lang;
    if( !isset( $this->aUsers ) ){
      $oFFS = FlatFilesSerialize::getInstance( );
      $this->aUsers = $oFFS->getData( DB_USERS, true );
    }

    if( isset( $GLOBALS['sPhrase'] ) )
      $aUsers = $this->generateUsersSearchListArray( $GLOBALS['sPhrase'] );
    else{
      if( isset( $this->aUsers ) && is_array( $this->aUsers ) && count( $this->aUsers ) > 0 ){
        foreach( $this->aUsers as $iUser => $aData ){
          $aUsers[] = $iUser;
        } // end foreach
      }
    }

    if( isset( $aUsers ) && is_array( $aUsers ) ){
      $this->countUsersOrders( );

      if( isset( $_GET['sSort'] ) && !empty( $_GET['sSort'] ) )
        $aUsers = $this->sortUsers( $aUsers, $_GET['sSort'] );

      $iCount = count( $aUsers );
      $aKeys = countPageNumber( $iCount, ( isset( $_GET['iPage'] ) ? $_GET['iPage'] : null ) );
      
      $content  = null;
      for( $i = $aKeys['iStart']; $i < $aKeys['iEnd']; $i++ ){
        $aList = $this->aUsers[$aUsers[$i]];
        
        $aList['iStyle']  = ( $i % 2 ) ? 0: 1;

        if( isset( $aList['iPriceGroup'] ) && isset( $this->aGroups[$aList['iPriceGroup']] ) ){
          $aList['sPriceGroup'] = $this->aGroups[$aList['iPriceGroup']]['sName'];
        }
        else
          $aList['sPriceGroup'] = $lang['Price_group_0'];

        $content .= '<tr class="l'.$aList['iStyle'].'">
          <td class="id">'.$aList['iUser'].'</td>
          <td class="name"><a href="?p=users-form&amp;iUser='.$aList['iUser'].'">'.$aList['sLastName'].' '.$aList['sFirstName'].'</a></td>
          <td>'.( isset( $aList['sLogin'] ) ? $aList['sLogin'] : ( isset( $aList['sFbId'] ) ? '- facebook.com -' : '&nbsp;' ) ).'</td>
          <td>'.$aList['sEmail'].'</td>
          <td>'.$aList['sPriceGroup'].'</td>
          <td>[ <strong>'.( isset( $aList['iOrders'] ) ? $aList['iOrders'] : 0 ).'</strong> ] <a href="?p=orders-list&amp;sPhrase=&amp;iStatus=&amp;iUser='.$aList['iUser'].'">'.$lang['more'].'</a></td>
          <td class="options">
            <a href="?p=users-form&amp;iUser='.$aList['iUser'].'"><img src="'.DIR_TEMPLATES.'admin/img/ico_edit.gif" alt="'.$lang['edit'].'" title="'.$lang['edit'].'" /></a>
            <a href="?p=users-delete&amp;iUser='.$aList['iUser'].'"  onclick="return del( );"><img src="'.DIR_TEMPLATES.'admin/img/ico_del.gif" alt="'.$lang['delete'].'" title="'.$lang['delete'].'" /></a>
          </td>
        </tr>';
      } // end for

      if( isset( $content ) ){
        $GLOBALS['sPages'] = countPagesClassic( $iCount, $GLOBALS['config']['admin_list'], $aKeys['iPageNumber'], changeUri( $_SERVER['REQUEST_URI'] ) );
        return $content;
      }
    }
  } // end function listUsers

  /**
  * Shows users select
  * @return string
  * @param int $iSelectedUser
  */
  public function listUsersSelect( $iSelectedUser = null ){
    if( !isset( $this->aUsers ) ){
      $oFFS = FlatFilesSerialize::getInstance( );
      $this->aUsers = $oFFS->getData( DB_USERS, true );
    }

    if( isset( $this->aUsers ) && is_array( $this->aUsers ) && count( $this->aUsers ) > 0 ){
      foreach( $this->aUsers as $iUser => $aData ){
        $aUsers[] = $iUser;
      } // end foreach
    }

    if( isset( $aUsers ) && is_array( $aUsers ) ){
      $aUsers = $this->sortUsers( $aUsers, 'name' );

      $content = null;
      foreach( $aUsers as $iUser ){
        $aList = $this->aUsers[$iUser];
        if( isset( $iSelectedUser ) && $iSelectedUser == $iUser )
          $sSelected = 'selected="selected"';
        else
          $sSelected = null;

        $content .= '<option value="'.$iUser.'" '.$sSelected.'>'.$aList['sLastName'].' '.$aList['sFirstName'].'</option>';
      } // end foreach
      return $content;
    }
  } // end function listUsersSelect

  /**
  * Sort users
  * @return array
  * @param array  $aProducts
  * @param string $sSort
  */
  public function sortUsers( $aUsers, $sSort = null ){
    $iCount = count( $aUsers );

    if( $sSort == 'price_group' ){
      $sFunctionSort = 'sort';
      $sKey = 'iPriceGroup';
    }
    elseif( $sSort == 'login' ){
      $sFunctionSort = 'sort';
      $sKey = 'sLogin';
    }
    elseif( $sSort == 'name' ){
      $sFunctionSort = 'sort';
    }
    elseif( $sSort == 'orders' ){
      $sFunctionSort = 'rsort';
      $sKey = 'iOrders';
    }
    else{
      $sFunctionSort = 'rsort';
      $sKey = 'iUser';    
    }

    for( $i = 0; $i < $iCount; $i++ ){
      if( $sSort == 'price_group' )
        $mValue = ( isset( $this->aUsers[$aUsers[$i]][$sKey] ) && isset( $this->aGroups[$this->aUsers[$aUsers[$i]][$sKey]] ) ) ? $this->aGroups[$this->aUsers[$aUsers[$i]][$sKey]]['sName'] : null;
      elseif( $sSort == 'name' )
        $mValue = $this->aUsers[$aUsers[$i]]['sLastName'].' '.$this->aUsers[$aUsers[$i]]['sFirstName'] ;
      else
        $mValue = isset( $this->aUsers[$aUsers[$i]][$sKey] ) ? $this->aUsers[$aUsers[$i]][$sKey] : null;
      $aSort[$i][0] = $mValue;
      $aSort[$i][1] = $aUsers[$i];
    } // end for

    $sFunctionSort( $aSort );

    for( $i = 0; $i < $iCount; $i++ ){
      $aUsers[$i] = $aSort[$i][1];
    } // end for    
    return $aUsers;
  } // end function sortUsers 

  /**
  * Return array with users
  * @return array
  * @param string $sPhrase
  */
  public function generateUsersSearchListArray( $sPhrase ){
    if( isset( $this->aUsers ) ){
      $aWords = getWordsFromPhrase( $sPhrase );
      $iCount = isset( $aWords ) ? count( $aWords ) : 0;
      foreach( $this->aUsers as $iUser => $aData ){
        $iFound = 0;
        $sSearchData = implode( ' ', $aData );

        for( $i = 0; $i < $iCount; $i++ ){
          if( preg_match( '/'.$aWords[$i].'/ui', $sSearchData ) )
            $iFound++;
        } // end for

        if( $iFound == $iCount ){
          $aUsers[] = $iUser;
        }
      }

      if( isset( $aUsers ) )
        return $aUsers;
    }
  } // end function generateUsersSearchListArray

  /**
  * Shows groups list select
  * @return string
  * @param array $aSelected
  */
  public function listGroupsSelect( $aSelected = null, $aLang = LANGUAGE, $bParentsOnly = null ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    $content = null;
    if( isset( $aUsersGroups ) && is_array( $aUsersGroups ) && count( $aUsersGroups ) > 0 ){
      foreach( $aUsersGroups as $iGroup => $aGroup ){
        if( ( !isset( $aLang ) || $aLang == $aGroup['sLang'] ) && ( !isset( $bParentsOnly ) || !isset( $aGroup['iPriceGroup'] ) ) ){
          $sSelected = isset( $aSelected[$iGroup] ) ? ' selected="selected"' : null;
          if( !isset( $aLang ) )
            $aGroup['sName'] = '('.$aGroup['sLang'].') '.$aGroup['sName'];
          $content .= '<option value="'.$iGroup.'"'.$sSelected.'>'.$aGroup['sName'].'</option>';
        }
      } // end for
    }
    return $content;
  } // end function listGroupsSelect

  /**
  * Deletes user and his pages
  * @return void
  * @param int  $iUser
  */
  public function deleteUser( $iUser ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, true );
    if( isset( $aUsers ) && is_array( $aUsers ) && isset( $aUsers[$iUser] ) ){
      unset( $aUsers[$iUser] );
      $oFFS->saveData( DB_USERS, $aUsers );
    }
    $this->saveRelatedToGroups( $iUser );
  } // end function deleteUser

  /**
  * Shows users groups list
  * @return string
  * @param string $sLang
  */
  public function listUsersGroups( $sLang = null ){
    global $lang;
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );

    $content  = null;
    $i = 0;
    if( isset( $aUsersGroups ) && is_array( $aUsersGroups ) && count( $aUsersGroups ) > 0 ){
      foreach( $aUsersGroups as $iGroup => $aList ){
        if( !isset( $sLang ) || $aList['sLang'] == $sLang ){
          $aList['sName'] = changeTxt( $aList['sName'] );

          $aList['iStyle']  = ( $i % 2 ) ? 0: 1;

          $content .= '<tr class="l0">
            <td class="id">'.$aList['iGroup'].'</td>
            <td><a href="?p=users-groupForm&amp;iGroup='.$aList['iGroup'].'">'.$aList['sName'].'</a></td>
            <td>'.( ( isset( $aList['fDiscount'] ) ) ? $aList['fDiscount'].' %' : '-' ).'</td>
            <td>'.( ( isset( $aList['iPriceGroup'] ) && isset( $aUsersGroups[$aList['iPriceGroup']] ) ) ? $aUsersGroups[$aList['iPriceGroup']]['sName'] : '-' ).'</td>
            <td class="options">
              <a href="?p=users-groupForm&amp;iGroup='.$aList['iGroup'].'"><img src="'.DIR_TEMPLATES.'admin/img/ico_edit.gif" alt="'.$lang['edit'].'" title="'.$lang['edit'].'" /></a>
              <a href="?p=users-groupDelete&amp;iGroup='.$aList['iGroup'].'"  onclick="return del( );"><img src="'.DIR_TEMPLATES.'admin/img/ico_del.gif" alt="'.$lang['delete'].'" title="'.$lang['delete'].'" /></a>
            </td>
          </tr>';
          $i++;
        }
      } // end for
    }

    if( isset( $content ) )
      return '<table id="list" cellspacing="1">
      <thead>
        <tr class="l1">
          <td class="id">'.$lang['Id'].'</td>
          <td class="name">'.$lang['Name'].'</td>
          <td>'.$lang['Discount'].'</td>
          <td>'.$lang['Price_group'].'</td>
          <td class="options">&nbsp;</td>
        </tr>
      </thead>
      <tbody>'.$content.'</tbody></table>';
  } // end function listUsersGroups

  /**
  * Throws users group data
  * @return array
  * @param int  $iGroup
  */
  public function throwGroup( $iGroup ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    if( isset( $aUsersGroups[$iGroup] ) && isset( $aUsersGroups[$iGroup]['sLang'] ) && $aUsersGroups[$iGroup]['sLang'] == LANGUAGE ){
      $aUsersGroups[$iGroup]['sName'] = changeTxt( $aUsersGroups[$iGroup]['sName'] );
      return $aUsersGroups[$iGroup];
    }
  } // end function throwGroup

  /**
  * Deletes users group
  * @return void
  * @param int $iGroup
  */
  public function deleteUsersGroup( $iGroup ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    if( isset( $aUsersGroups ) && is_array( $aUsersGroups ) && isset( $aUsersGroups[$iGroup] ) ){
      unset( $aUsersGroups[$iGroup] );
      $oFFS->saveData( DB_USERS_GROUPS, $aUsersGroups );
    }
  } // end function deleteUsersGroup

  /**
  * Saves group
  * @return int
  * @param array  $aForm
  */
  public function saveGroup( $aForm ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );

    if( !is_numeric( $aForm['iGroup'] ) )
      $aForm['iGroup'] = $oFFS->throwLastId( DB_USERS_GROUPS, 'iGroup' ) + 1;

    if( isset( $aForm['iPriceGroup'] ) && !is_numeric( $aForm['iPriceGroup'] ) )
      unset( $aForm['iPriceGroup'] );

    if( isset( $aForm['fDiscount'] ) ){
      $aForm['fDiscount'] = trim( $aForm['fDiscount'] );
      if( !is_numeric( $aForm['fDiscount'] ) )
        unset( $aForm['fDiscount'] );
    }

    $aForm['sName']  = changeTxt( $aForm['sName'] );
    $aForm['sLang'] = LANGUAGE;

    $aForm['aPages'] = isset( $aUsersGroups[$aForm['iGroup']]['aPages'] ) ? $aUsersGroups[$aForm['iGroup']]['aPages'] : null;
    $aForm['aUsers'] = isset( $aUsersGroups[$aForm['iGroup']]['aUsers'] ) ? $aUsersGroups[$aForm['iGroup']]['aUsers'] : null;
    $aUsersGroups[$aForm['iGroup']] = compareArrays( $GLOBALS['aUsersGroupsFields'], changeMassTxt( $aForm, '' ) );

    $oFFS->saveData( DB_USERS_GROUPS, $aUsersGroups );
    return $aForm['iGroup'];
  } // end function saveGroup

  /**
  * Saves related groups to user or pages
  * @return void
  * @param string $sLang
  */
  public function deleteGroupsFromLanguage( $sLanguage ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    if( isset( $aUsersGroups ) && is_array( $aUsersGroups ) && count( $aUsersGroups ) > 0 ){
      foreach( $aUsersGroups as $iGroup => $aGroup ){
        if( $aGroup['sLang'] == $sLanguage ){
          unset( $aUsersGroups[$iGroup] );
          $bChanged = true;
        }
      } // end for
    }
    if( isset( $bChanged ) )
      $oFFS->saveData( DB_USERS_GROUPS, $aUsersGroups );
  } // end function deleteGroupsFromLanguage

  /**
  * List last added users
  * @return string
  */
  public function listLastUsers( ){

    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, true );

    if( isset( $aUsers ) && is_array( $aUsers ) && count( $aUsers ) > 0 ){
      foreach( $aUsers as $iUser => $aData ){
        if( !isset( $aData['iTime'] ) )
          $aData['iTime'] = 1000000000;
        $aUsersTime[$aData['iTime']] = $iUser;
      } // end foreach
      krsort( $aUsersTime );

      $iMax = 5;
      $i = 0;
      $content = null;
      foreach( $aUsersTime as $iUser ){
        $aData = $aUsers[$iUser];
        $content .= '<tr><td class="id">'.$aData['iUser'].'</td><td class="name"><a href="?p=users-form&amp;iUser='.$iUser.'">'.$aData['sLastName'].' '.$aData['sFirstName'].'</a></td><td class="data">'.( isset( $aData['iTime'] ) ? displayDate( $aData['iTime'] ) : '-' ).'</td></tr>';
        $i++;

        if( $i >= $iMax )
          break;
      } // end foreach

      return '<table cellspacing="0"><thead><tr><td>'.$GLOBALS['lang']['Id'].'</td><td>'.$GLOBALS['lang']['Last_name'].', '.$GLOBALS['lang']['First_name'].'</td><td>'.$GLOBALS['lang']['Date'].'</td></tr></thead><tbody>'.$content.'</tbody></table>';
    }
  } // end function listLastUsers

  /**
  * Counts users orders
  * @return void
  */
  private function countUsersOrders( ){
    $oOrder = new OrdersAdmin( );
    $oOrder->generateCache( );
    if( isset( $oOrder->aOrders ) && is_array( $oOrder->aOrders ) && count( $oOrder->aOrders ) > 0 ){
      foreach( $oOrder->aOrders as $aData ){
        if( isset( $aData['iUser'] ) && $aData['iUser'] > 0 ){
          if( isset( $this->aUsers[$aData['iUser']]['iOrders'] ) )
            $this->aUsers[$aData['iUser']]['iOrders']++;
          else
            $this->aUsers[$aData['iUser']]['iOrders'] = 1;
        }
      } // end foreach
    }
  } // end function countUsersOrders

}
?>