<?php
class Users{

  private static $oInstance = null;
  public $aGroups = null;
  public $aPagesHidden = null;
  public $aGroupPages = null;
  public $aUserGroups = null;

  public static function getInstance( $config = null ){  
    if( !isset( self::$oInstance ) ){  
      self::$oInstance = new Users( $config );  
    }  
    return self::$oInstance;  
  } // end function getInstance

  /**
  * Constructor
  * @return void
  */
  private function __construct( $config = null ){
    $this->generateCache( $config );
  } // end function __construct

 /**
  * Generate cache variables
  * @return void
  */
  public function generateCache( $config = null ){
    if( defined( 'CUSTOMER_PAGE' ) && throwStatus( ) == 1 ){
      $oFFS = FlatFilesSerialize::getInstance( );
      $this->aGroups = $oFFS->getData( DB_USERS_GROUPS, true );
      if( isset( $this->aGroups ) && is_array( $this->aGroups ) && count( $this->aGroups ) > 0 ){
        foreach( $this->aGroups as $iGroup => $aGroup ){
          if( !isset( $aGroup['sLang'] ) || $aGroup['sLang'] != LANGUAGE ){
            unset( $this->aGroups[$iGroup], $aGroup['sLang'] );
          }
          if( isset( $aGroup['sLang'] ) && $aGroup['sLang'] == LANGUAGE && isset( $aGroup['aPages'] ) && is_array( $aGroup['aPages'] ) && count( $aGroup['aPages'] ) > 0 ){
            foreach( $aGroup['aPages'] as $iPage => $sValue ){
              $this->aPagesHidden[$iPage] = true;
              $this->aGroupPages[$iGroup][$iPage] = $iPage;
            } // end foreach
          }
        } // end foreach
      }

      if( isset( $_SESSION['iUserClient'] ) && is_numeric( $_SESSION['iUserClient'] ) ){
        $this->aUserGroups = $this->throwRelatedGroups( $_SESSION['iUserClient'] );
        if( isset( $this->aUserGroups ) && is_array( $this->aUserGroups ) && count( $this->aUserGroups ) > 0 ){
          foreach( $this->aUserGroups as $iGroup ){
            if( isset( $this->aGroupPages[$iGroup] ) && is_array( $this->aGroupPages[$iGroup] ) && count( $this->aGroupPages[$iGroup] ) > 0 ){
              foreach( $this->aGroupPages[$iGroup] as $iPage ){
                unset( $this->aPagesHidden[$iPage] );
              } // end foreach
            }
          } // end foreach
        }
        if( !empty( $config['login_page'] ) )
          $this->aPagesHidden[$config['login_page']] = true;
        if( !empty( $config['register_page'] ) )
          $this->aPagesHidden[$config['register_page']] = true;
        if( !empty( $config['forget_password_page'] ) )
          $this->aPagesHidden[$config['forget_password_page']] = true;
      }
      else{
        if( !empty( $config['logout_page'] ) )
          $this->aPagesHidden[$config['logout_page']] = true;
        if( !empty( $config['profile_page'] ) )
          $this->aPagesHidden[$config['profile_page']] = true;
        if( !empty( $config['user_orders_page'] ) )
          $this->aPagesHidden[$config['user_orders_page']] = true;
      }
    }
  } // end function generateCache

  /**
  * Sets start variables for user
  */
  public function setUserVariables( $aUser = null ){
    if( isset( $aUser ) && is_array( $aUser ) ){
      $_SESSION['iUserClient'] = $aUser['iUser'];
      $_SESSION['sUserName'] = $aUser['sFirstName'].' '.$aUser['sLastName'];
      $_SESSION['sUserEmail'] = $aUser['sEmail'];
      $_SESSION['sUserPhone'] = isset( $aUser['sPhone'] ) ? $aUser['sPhone'] : null;
      $_SESSION['iUserPriceGroup'] = $_SESSION['fDiscount'] = null;
      if( isset( $aUser['iPriceGroup'] ) && is_numeric( $aUser['iPriceGroup'] ) && isset( $this->aGroups[$aUser['iPriceGroup']] ) ){
        $_SESSION['iUserPriceGroup'] = ( isset( $this->aGroups[$aUser['iPriceGroup']]['iPriceGroup'] ) ) ? $this->aGroups[$aUser['iPriceGroup']]['iPriceGroup'] : $this->aGroups[$aUser['iPriceGroup']]['iGroup'];
        if( isset( $this->aGroups[$_SESSION['iUserPriceGroup']]['fDiscount'] ) )
          $_SESSION['fDiscount'] = $this->aGroups[$_SESSION['iUserPriceGroup']]['fDiscount'] / 100;
      }
    }
    else{
      unset( $_SESSION['iUserClient'], $_SESSION['sUserName'], $_SESSION['sUserEmail'], $_SESSION['sUserPhone'], $_SESSION['iUserPriceGroup'], $_SESSION['fDiscount'] );
      if( defined( 'FB_APP_ID' ) && isset( $_SESSION['fb_'.FB_APP_ID.'_user_id'] ) )
        unset( $_SESSION['fb_'.FB_APP_ID.'_user_id'], $_SESSION['fb_'.FB_APP_ID.'_access_token'], $_SESSION['fb_'.FB_APP_ID.'_code'] );
    }
  } // end function setUserVariables

  /**
  * Throws related groups to user or page
  * @return void
  * @param int $iRelated
  * @param string $sKey
  */
  public function throwRelatedGroups( $iRelated, $sKey = 'Users' ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsersGroups = $oFFS->getData( DB_USERS_GROUPS, true );
    $aReturn = null;

    if( isset( $aUsersGroups ) && is_array( $aUsersGroups ) && count( $aUsersGroups ) > 0 ){
      foreach( $aUsersGroups as $iGroup => $aGroup ){
        if( isset( $aGroup['a'.$sKey][$iRelated] ) )
          $aReturn[$iGroup] = $iGroup;
      } // end for
    }
    return $aReturn;
  } // end function throwRelatedGroups

  /**
  * Saves user
  * @return int
  * @param array  $aForm
  */
  public function saveUser( $aForm, $iUser = null ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, true );

    if( isset( $iUser ) )
      $aForm['iUser'] = $iUser;

    if( !empty( $aForm['sLogin'] ) )
      $aForm['sLogin'] = trim( $aForm['sLogin'] );

    if( isset( $aForm['iUser'] ) && is_numeric( $aForm['iUser'] ) ){
      $aData  = $aUsers[$aForm['iUser']];
      $aForm['iTime']  = $aData['iTime'];
      if( !isset( $aForm['sLogin'] ) || empty( $aForm['sLogin'] ) ){ // profile
        if( isset( $aData['sLogin'] ) )
          $aForm['sLogin'] = $aData['sLogin'];
        if( isset( $aData['iPriceGroup'] ) && is_numeric( $aData['iPriceGroup'] ) )
          $aForm['iPriceGroup'] = $aData['iPriceGroup'];
        if( isset( $aData['sFbId'] ) )
          $aForm['sFbId'] = $aData['sFbId'];
      }

      if( isset( $aForm['sPass'] ) && !empty( $aForm['sPass'] ) )
        $aForm['sPass'] = md5( $aForm['sPass'] );
      elseif( isset( $aData['sPass'] ) )
        $aForm['sPass'] = $aData['sPass'];
    }
    else{
      $aForm['iUser'] = $oFFS->throwLastId( DB_USERS, 'iUser' ) + 1;
      $aForm['iTime'] = time( );
      if( defined( 'CUSTOMER_PAGE' ) ){
        if( !empty( $GLOBALS['config']['default_users_group'] ) )
          $aForm['iPriceGroup'] = $GLOBALS['config']['default_users_group'];
        if( $GLOBALS['config']['user_send_email'] === true ){
          sendEmail( Array( 'sTopic' => $GLOBALS['lang']['Add_user'], 'sSender' => $GLOBALS['config']['email'], 'sMailContent' => $GLOBALS['lang']['New_user_registered']."\n".$GLOBALS['lang']['Id'].": ".$aForm['iUser']."\n".$GLOBALS['lang']['Email'].": ".changeTxt( $aForm['sEmail'] ) ), null );
        }
      }
    }

    $aUsers[$aForm['iUser']] = compareArrays( $GLOBALS['aUsersFields'], changeMassTxt( $aForm, '' ) );

    $oFFS->saveData( DB_USERS, $aUsers );

    return $aForm['iUser'];
  } // end function saveUser

  /**
  * Throws user data
  * @return array
  * @param int  $iUser
  * @param bool $bCache
  */
  public function throwUser( $iUser, $bCache = true ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, $bCache );
    if( isset( $aUsers ) && is_array( $aUsers ) && isset( $aUsers[$iUser] ) ){
      $aList = $aUsers[$iUser];
      if( !empty( $aList['sNip'] ) )
        $aList['sVrn'] = $aList['sNip'];
      $aList = changeMassTxt( $aList );
      return $aList;
    }
  } // end function throwUser

  /**
  * Throws user data
  * @return array
  * @param int  $sLogin
  */
  public function throwUserByLogin( $sLogin ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, true );
    if( isset( $aUsers ) && is_array( $aUsers ) && count( $aUsers ) > 0 ){
      foreach( $aUsers as $aData ){
        if( isset( $aData['sLogin'] ) && $aData['sLogin'] == $sLogin )
          return changeMassTxt( $aData );
      } // end for
    }
  } // end function throwUserByLogin

  /**
  * Checks user login and password
  * @return int
  * @param string $sLogin
  * @param string $sPass
  */
  public function checkUserLogin( $sLogin, $sPass ){
    $sLogin = changeTxt( $sLogin );
    $sPass = changeTxt( $sPass );
    $sPassMd5 = md5( $sPass );
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, true );

    if( isset( $aUsers ) && is_array( $aUsers ) && count( $aUsers ) > 0  ){
      foreach( $aUsers as $aUser ){
        if( isset( $aUser['sLogin'] ) && $aUser['sLogin'] == $sLogin && ( ( strlen( $aUser['sPass'] ) == 32 && $aUser['sPass'] == $sPassMd5 ) || ( strlen( $aUser['sPass'] ) != 32 && $aUser['sPass'] == $sPass ) ) && isset( $aUser['iUser'] ) && is_numeric( $aUser['iUser'] ) ){
          $this->setUserVariables( $this->throwUser( $aUser['iUser'] ) );
          return true;
        }
      } // end foreach
    }
    return null;
  } // end function checkUserLogin

  /**
  * Checks user from Facebook
  * @return int
  * @param array $aFbUser
  */
  public function loginFbUser( $aFbUser ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, true );

    if( isset( $aUsers ) && is_array( $aUsers ) && count( $aUsers ) > 0  ){
      foreach( $aUsers as $aUser ){
        if( isset( $aUser['sFbId'] ) && $aUser['sFbId'] == $aFbUser['id'] && isset( $aUser['iUser'] ) && is_numeric( $aUser['iUser'] ) ){
          $this->setUserVariables( $this->throwUser( $aUser['iUser'] ) );
          return true;
        }
      } // end foreach
    }
    $iAdded = $this->saveUser( Array( 'sFirstName' => $aFbUser['first_name'], 'sLastName' => $aFbUser['last_name'], 'sEmail' => $aFbUser['email'], 'sFbId' => $aFbUser['id'] ) );
    if( isset( $iAdded ) && is_numeric( $iAdded ) ){
      $this->setUserVariables( $this->throwUser( $iAdded, null ) );
      return true;
    }
    return null;
  } // end function loginFbUser

  /**
  * Checks if users login not exist
  * @return int
  * @param string $sLogin
  */
  public function checkUserLoginNotExist( $sLogin ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aUsers = $oFFS->getData( DB_USERS, true );
    if( isset( $aUsers ) && is_array( $aUsers ) && count( $aUsers ) > 0 ){
      $sLogin = changeTxt( trim( $sLogin ) );
      foreach( $aUsers as $aUser ){
        if( isset( $aUser['sLogin'] ) && $aUser['sLogin'] == $sLogin )
          return null;
      } // end foreach
    }
    return true;
  } // end function checkUserLoginNotExist

  /**
  * Checks if user data in login form are correct
  * @return bool
  * @param array $aData
  */
  function checkUserDataBeforeOrder( $aData ){
    if( !empty( $aData['sPass'] ) && !empty( $aData['sPass2'] ) ){
      if( $aData['sPass'] != $aData['sPass2'] )
        return null;
      elseif( $this->checkUserLoginNotExist( $aData['sLogin'] ) === true ){
        define( 'CHECK_USER_DATA', true );
        return true;
      }
      else{
        define( 'CHECK_USER_DATA', false );
        return false;
      }
    }
  } // end function checkUserDataBeforeOrder

}
?>